@@ -26,43 +26,52 @@ class SettingsPage(Gtk.Box):
         self.set_margin_top(15)
         self.set_margin_bottom(15)
 
-    def add_section(self, title):
-        section = SettingsBox(title)
+    def add_section(self, title=None):
+        section = SettingsSection(title)
         self.pack_start(section, False, False, 0)
 
         return section
 
-class SettingsBox(Gtk.Frame):
+class SettingsSection(Gtk.Box):
+    def __init__(self, title=None):
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
+        self.set_spacing(10)
 
-    def __init__(self, title):
-        Gtk.Frame.__init__(self)
-        self.set_shadow_type(Gtk.ShadowType.IN)
-        frame_style = self.get_style_context()
+        if title:
+            header_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+            header_box.set_spacing(5)
+            self.add(header_box)
+            label = Gtk.Label()
+            label.set_markup("<b>%s</b>" % title)
+            label.set_alignment(0, 0.5)
+            header_box.add(label)
+
+        self.frame = Gtk.Frame()
+        self.frame.set_shadow_type(Gtk.ShadowType.IN)
+        frame_style = self.frame.get_style_context()
         frame_style.add_class("view")
         self.size_group = Gtk.SizeGroup()
         self.size_group.set_mode(Gtk.SizeGroupMode.VERTICAL)
 
         self.box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self.add(self.box)
+        self.frame.add(self.box)
 
-        toolbar = Gtk.Toolbar.new()
-        toolbar_context = toolbar.get_style_context()
-        Gtk.StyleContext.add_class(Gtk.Widget.get_style_context(toolbar), "cs-header")
+        self.need_separator = False
 
-        label = Gtk.Label.new()
-        label.set_markup("<b>%s</b>" % title)
-        title_holder = Gtk.ToolItem()
-        title_holder.add(label)
-        toolbar.add(title_holder)
-        self.box.add(toolbar)
+    def add_row(self, widget):
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        if self.need_separator:
+            vbox.add(Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL))
+        list_box = Gtk.ListBox()
+        list_box.set_selection_mode(Gtk.SelectionMode.NONE)
+        list_box.add(widget)
+        vbox.add(list_box)
+        self.box.add(vbox)
 
-        self.list_box = Gtk.ListBox()
-        self.list_box.set_selection_mode(Gtk.SelectionMode.NONE)
-        self.list_box.set_header_func(list_header_func, None)
-        self.box.add(self.list_box)
+        if self.frame.get_parent() is None:
+            self.add(self.frame)
 
-    def add_row(self, row):
-        self.list_box.add(row)
+        self.need_separator = True
 
 
 class SettingsRow(Gtk.ListBoxRow):
@@ -318,6 +327,65 @@ class SettingsCombo(Gtk.ComboBox):
             self.keyfile.set_string(GROUP_NAME, self.key, value)
             self.keyfile.save_to_file(CONF_PATH)
 
+class SettingsComboMousePointer(Gtk.ComboBox):
+    def __init__(self, keyfile, key, backup_filename, options, valtype="string", size_group=None):
+        self.key = key
+        self.keyfile = keyfile
+        self.backup_filename = backup_filename
+        try:
+            self.value = keyfile.get_string("Icon Theme", key)
+        except Exception:
+            self.value = "DMZ-White"
+        Gtk.ComboBox.__init__(self)
+
+        if size_group:
+            size_group.add_widget(self)
+
+        renderer_text = Gtk.CellRendererText()
+        self.pack_start(renderer_text, True)
+        self.add_attribute(renderer_text, "text", 1)
+        self.set_valign(Gtk.Align.CENTER)
+
+        # assume all keys are the same type (mixing types is going to cause an error somewhere)
+        var_type = type(options[0][0]) if options else None
+        self.model = Gtk.ListStore(var_type, str)
+        self.valtype = valtype
+        self.option_map = {}
+        for option in options:
+            self.option_map[option[0]] = self.model.append([option[0], option[1]])
+
+        self.set_model(self.model)
+        self.set_id_column(0)
+
+        if self.value in self.option_map.keys():
+            self.set_active_iter(self.option_map[self.value])
+
+        self.connect("changed", self.on_changed)
+
+    def on_changed(self, widget):
+        tree_iter = widget.get_active_iter()
+        if tree_iter != None:
+            value = self.model[tree_iter][0]
+            self.keyfile.set_string("Icon Theme", self.key, value)
+            self.keyfile.save_to_file(self.backup_filename)
+
+class SettingsEntry(Gtk.Entry):
+    def __init__(self, keyfile, settings, key):
+        self.key = key
+        self.keyfile = keyfile
+        try:
+            self.value = keyfile.get_string(GROUP_NAME, key)
+        except:
+            self.value = settings.get_string(key)
+
+        Gtk.Entry.__init__(self)
+        self.set_text(self.value)
+        self.connect("changed", self.on_changed)
+
+    def on_changed(self, widget, data=None):
+        self.keyfile.set_string(GROUP_NAME, self.key, self.get_text())
+        self.keyfile.save_to_file(CONF_PATH)
+
 class LightDMSwitch(Gtk.Switch):
     def __init__(self, keyfile, key, value):
         self.key = key
